%% Kyvadlo na pruzine (Pendolum with spring)
%  Computation of equations for system modeling
%
%  Date: 23.11.20011
%  Author: Vladimir Petrik
%  Author: Pavel Krsek

%% Variable Initialization
clear all;close all;clc;
syms t G m l0 K;

% Indipendent variables
s = [
        sym('l(t)');
        sym('alpha(t)');
        sym('betta(t)');
    ];
% Hanging position
p = [
        sym('xp(t)');
        sym('yp(t)');
        sym('zp(t)');
    ];

%% Forces and Accelerations
l = s(1);
alpha = s(2);
beta = s(3);
% Bounding equations
pos = [ 
        l * cos(beta) * cos(alpha) + p(1);
        l * sin(beta) + p(2);
       -l * cos(beta) * sin(alpha) + p(3);
      ];
% Second derivatives of bounding equations  
acc = diff(pos,t,2);
% Force components
T = K*(l-l0);
F = [
        T*cos(beta)*cos(alpha);
        T*sin(beta);
        G - T*cos(beta)*sin(alpha);
    ];
% Newton equation
eqn = m*acc + F;

%% Solving of equqtions
% Newton equation can be rewriten into form Aw = b
% where w = diff(s,t,2)
[A b] = axb(eqn,diff(s,t,2));

% Solwing of the equations w = inv(A) * b
AI = inv(A);
AI = simplify(AI);
w = AI*b;
w = simplify(w);
% Alternative solution
% fun = A\b;

%% Substitution of time dependent variables f(t) 
%  The functions f(t) are replaced by names of the variables
%  in matlab execution code (zdot.m)

% Second derivatives of hanging position
pdd = {'xpdd';'ypdd';'zpdd'};
% First derivatives of indipendent variables
ds  = {'ld';'ad';'bd'};
% Indipendent variables
st  = {'l';'a';'b'};

% Preparing substitution
j = 0;
for i = 1:length(p)
    OLD{j+i} = diff(p(i),t,2);
    NEW{j+i} = sym(pdd{i});
end
j = length(OLD);
for i = 1:length(s)
    OLD{j+i*2-1} = diff(s(i),t,1);
    NEW{j+i*2-1} = sym(ds{i});
    OLD{j+i*2} = s(i);
    NEW{j+i*2} = sym(st{i});
end

sbsA = subs(A,OLD,NEW,0)
sbsb = subs(b,OLD,NEW,0)
sbsw = subs(w,OLD,NEW,0)

%% Creating Matlab function
% Not working with:
%   MATLAB Version 7.6.0.324 (R2008a)
%   Symbolic Math Toolbox Version 3.2.3
% ht = matlabFunction(sbsw,'file','funct.m');
% disp('Function have been created, See help funct');



